# ../addons/eventscripts/gungame/scripts/custom/gg_buylevel/gg_buylevel.py

# ============================================================================
# >> IMPORTS
# ============================================================================
# Eventscripts Imports
import es
from cmdlib import registerSayCommand
from cmdlib import unregisterSayCommand
import gamethread
from playerlib import getPlayer

# GunGame Imports
from gungame51.core.addons.shortcuts import AddonInfo
from gungame51.core.addons import PriorityAddon
from gungame51.core.players.shortcuts import Player
from gungame51.core.players.shortcuts import setAttribute
from gungame51.core.players.shortcuts import deleteAttribute
from gungame51.core.messaging.shortcuts import msg
from gungame51.core.weapons.shortcuts import get_total_levels
from gungame51.scripts.included.gg_turbo.gg_turbo import give_weapon

# ============================================================================
# >> ADDON REGISTRATION/INFORMATION
# ============================================================================
info = AddonInfo()
info.name = "gg_buylevel"
info.title = "GG BuyLevel"
info.author = "Warren Alpert"
info.version = "0.9"
info.translations = ["gg_buylevel"]

# ============================================================================
# >> GLOBAL VARIABLES
# ============================================================================
gg_buylevel_cmd = str(es.ServerVar("gg_buylevel_cmd"))
gg_buylevel_kill_reward = es.ServerVar("gg_buylevel_kill_reward")
gg_buylevel_levelup_reward = es.ServerVar("gg_buylevel_levelup_reward")
gg_buylevel_cost = es.ServerVar("gg_buylevel_cost")
# Make a list of weapons
gg_buylevel_block_skip = str(es.ServerVar("gg_buylevel_block_skip")).split(",")
gg_buylevel_announce = es.ServerVar("gg_buylevel_announce")

# Be enabled by default
enabled = True

# A list to store those who bought a level so that we know not to reward them
# gg_levelup fires
recentlyBought = []

# ============================================================================
# >> LOAD & UNLOAD
# ============================================================================
def load():
    es.dbgmsg(0, "Loaded: %s" % info.name)

    # Set everyone's cash to 0
    for userid in es.getUseridList():
        set_cash(userid, 0)

    # Register the say command
    registerSayCommand(gg_buylevel_cmd, buy_level_cmd, "Allows players to " +
                                                        "buy levels.")

def unload():
    es.dbgmsg(0, "Unloaded: %s" % info.name)

    # Delete the custom player attribute
    deleteAttribute("#all", "cash")

    # Unregister the say command
    unregisterSayCommand(gg_buylevel_cmd)
    
# ============================================================================
# >> GAME EVENTS
# ============================================================================
def buy_level_cmd(userid, args):
    # If gg_buylevel is disabled, or a PriorityAddon (primarily warmup) is
    # loaded, stop here
    if not enabled or PriorityAddon():
        msg(userid, "BuyLevel_NotStarted", prefix=True)
        return

    ggPlayer = Player(userid)
    
    # If the player's current weapon is in the block skipping list, or the
    # player is on the last level, stop here
    if ggPlayer.weapon in gg_buylevel_block_skip or ggPlayer.level == \
                                                            get_total_levels():
        msg(userid, "BuyLevel_BlockSkip", {"weapon":ggPlayer.weapon},
                                                                        True)
        return

    cash = getPlayer(userid).cash

    # If the player does not have enough money, stop here
    if getPlayer(userid).cash < int(gg_buylevel_cost):
        msg(userid, "BuyLevel_NotEnoughCash", {"cost":int(gg_buylevel_cost)},
                                                                        True)
        return

    # If preventlevel is enabled for the player, stop here
    if ggPlayer.preventlevel:
        msg(userid, "BuyLevel_PreventLevel", prefix=True)
        return

    # Add the userid to recentlyBought so that when gg_levelup fires we will
    # know not to reward them
    recentlyBought.append(userid)
    
    # In case gg_levelup fails to fire, remove the userid from recentlyBought
    gamethread.delayed(1, remove_buyer, userid)
    
    # Remove the money to pay for the level
    set_cash(userid, cash - gg_buylevel_cost)
    # Level up the player
    ggPlayer.levelup(1, reason="gg_buylevel")
    ggPlayer.playsound("levelup")


    # Turbo will handle giving the new weapon, but if it isn't loaded, give it
    # ourselves
    if not int(es.ServerVar("gg_turbo")):
        give_weapon(userid, int(ggPlayer.level) - 1)

def player_disconnect(event_var):
    # Players may disconnect before activating, causing an error
    try:
        # Delete the custom player attribute
        deleteAttribute(int(event_var["userid"]), "cash")
    except:
        return

def player_spawn(event_var):
    userid = int(event_var["userid"])

    # If the player has not spawned yet
    if not hasattr(Player(userid), "cash"):
        # Set their cash to 0
        set_cash(userid, 0)
        
        # If gg_buylevel_announce is enabled, message the player
        if int(gg_buylevel_announce):
            msg(userid, "BuyLevel_Announcement", {"command":gg_buylevel_cmd,
                                        "cost":int(gg_buylevel_cost)}, True)
        return

    # If the player has spawned already, set the player's cash
    set_cash(userid, Player(userid).cash)

def gg_levelup(event_var):
    attacker = int(event_var["attacker"])

    # If this gg_levelup was triggered by a purchased level, stop here
    if attacker in recentlyBought:
        recentlyBought.remove(attacker)
        return

    # If gg_buylevel is disabled, stop here
    if not enabled:
        return

    # If the attacker is still on the server, increment the player's cash
    if es.exists('userid', attacker):
        set_cash(attacker, getPlayer(attacker).cash +
                                int(gg_buylevel_levelup_reward))

def player_death(event_var):
    # If gg_buylevel is disabled, stop here
    if not enabled:
        return

    # Get the player's cash
    attacker = int(event_var["attacker"])

    # If there is no attacker, stop here
    if not attacker:
        return

    cash = getPlayer(attacker).cash

    # If it was a teamkill, stop here
    if event_var["es_userteam"] == event_var["es_attackerteam"]:
        return

    # Increment the player's cash (subtracting the 300 which valve rewards for
    # a kill)
    set_cash(attacker, cash + int(gg_buylevel_kill_reward) - 300)

def es_map_start(event_var):
    global enabled

    # Set everyone's cash to 0
    setAttribute("#all", "cash", 0)

    # Disable this addon until the next gg_start (For the first time the server
    # starts)
    enabled = False

def gg_win(event_var):
    global enabled

    # Disable this addon until the next gg_start
    enabled = False

def gg_start(event_var):
    global enabled

    # Set everyone's cash to 0
    for userid in es.getUseridList():
        set_cash(userid, 0)

    # Enable gg_buylevel
    enabled = True

# ============================================================================
# >> CUSTOM/HELPER FUNCTIONS
# ============================================================================
def remove_buyer(userid):
    # Remove the level buyer if they were not removed because gg_levelup didn't
    # fire
    if userid in recentlyBought:
        recentlyBought.remove(userid)

def set_cash(userid, value):
    # Limit the cash value to 0-16000
    if value < 0:
        value = 0
    elif value > 16000:
        value = 16000

    # Set the player's cash, and update their GunGame player attribute
    getPlayer(userid).cash = value
    setAttribute(userid, "cash", value)